function [bvp,info]=set_sBVP02_sphere_map(N,varargin)
% [bvp,info]=fc_vfemp1.addon.surface.samples.set_sBVP02_sphere_map(N,varargin)
%   Sets the surface BVP on the unit sphere mapping by Fmap function
%       - LaplaceBeltrami(u) + <grad_Gamma(u), c > + a0 u = f    on Gamma
%
%   The default sBVP is print by using 
%        set_sBVP_sphere_map(0)
%
% OUTPUT
%   bvp, a BVP object which contatins the description of the sBVP to solve.
%   info, a structure which contains some informations!
%
% INPUTS
%   N is an integer used for mesh refinement: bigger N gives bigger mesh file.
%   (see sphere4surf.geo file)
%   
% USAGE
%   [bvp,info]=set_sBVP_sphere_map(N,key,value,...) allows additional key/value
%     pairs to be used. 
%     The key strings which can modify the sBVP to solve are
%     
%     'u'    Set exact solution as function handle @(x,y,z)... or empty if the
%            exact solution is unknown. Default is the function handle
%               @fc_vfemp1.addon.surface.samples.uxyz01
%     'f'    If 'u' is not empty then 'f' is set to the function handle which
%            contains -LaplaceBeltrami(u). The RHS term in the sBVP is then 
%            given by
%               RHS= f + a0*u
%            If 'u' is empty then 'f' is the RHS term in the sBVP given as 
%            a function handle or a scalar:
%               RHS= f
%            Default is the function handle
%               @fc_vfemp1.addon.surface.samples.fxyz01
%     'a0'   Set the a0 term in the sBVP as a function handle or as scalar.
%            Default is 
%                @(x,y,z) 1+x.^2
%     'Fmap' Set the mapping function as a function handle.
%            Default is the function handle
%               @fc_vfemp1.addon.surface.samples.Fmapxyz01
%            which is the identity 
%     'c'    Set the a0 term in the sBVP as a cell of 3 function handle.
%            Default is 
%                {@(x,y,z) cos(x), @(x,y,z) sin(y), @(x,y,z) 2+x.*y.*z}
%
%     The other key strings are
%
%     'verbose' Set verbosity level. Default is 0: no verbosity.
%     'geodir'  Set directory of the geofile 'sphere4surf.geo'
%     'meshdir' Set directory for the mesh files
%   
% SAMPLES
%   see sBVP02_sphere_map_01.m, sBVP02_sphere_map_ex01.m, sBVP02_sphere_map_ex02.m
  env=fc_vfemp1.addon.surface.environment();
  tostr=@(f) fc_tools.utils.fun2str(f);
  c_def= {@(x,y,z) cos(x), @(x,y,z) sin(y), @(x,y,z) 2+x.*y.*z};
  a0_def=@(x,y,z) 1+x.^2;
  p = inputParser;
  p.addParamValue('verbose',0,@isscalar);
  p.addParamValue('u',@(x,y,z)fc_vfemp1.addon.surface.samples.uxyz01(x,y,z),@(x) isempty(x) || strcmp(class(x),'function_handle'));
  p.addParamValue('f',@(x,y,z)fc_vfemp1.addon.surface.samples.fxyz01(x,y,z),@(x) isscalar(x) || strcmp(class(x),'function_handle'));
  p.addParamValue('Fmap',@(x,y,z)fc_vfemp1.addon.surface.samples.Fmapxyz01(x,y,z),@(x) strcmp(class(x),'function_handle'));
  p.addParamValue('a0',a0_def,@(x) isscalar(x) || strcmp(class(x),'function_handle'));
  p.addParamValue('gradSu',@(x,y,z)fc_vfemp1.addon.surface.samples.gradSuxyz01(x,y,z),@(x) isempty(x) || strcmp(class(x),'function_handle'));
  p.addParamValue('c',c_def,@iscell);
  p.addParamValue('geodir',env.geodir,@ischar); 
  p.addParamValue('meshdir',env.meshdir,@ischar);
  p.parse(varargin{:});
  R=p.Results;
  verbose=R.verbose;
  u=R.u;f=R.f;Fmap=R.Fmap;
  a0=R.a0;c=R.c;gradSu=R.gradSu;
  dim=3;d=2;
  name='sphere4surf';
  geofile=fullfile(R.geodir,name);
  if N==0, verbose=10*(N==0);end
    if verbose>=1
    fprintf('------------------------------------------------------\n'); 
    fprintf('      Surface BVP on unit sphere map by Fmap\n');
    fprintf('  with mesh generated from gmsh and %s.geo file\n\n',geofile);
    fprintf('         - Delta_Gamma u +<grad_Gamma u,c>+ a0 u = f\n');
    fprintf('          -> No boundary conditions\n')    
    fprintf('  f       : %s\n',tostr(f) );
    fprintf('  a0      : %s\n',tostr(a0) );
    for i=1:3,fprintf('  c{%d}    : %s\n',i,tostr(c{i}));end
    fprintf('  u exact : %s\n',tostr(u) );
    fprintf('  Fmap    : %s\n',tostr(Fmap) );
    fprintf('------------------------------------------------------\n');
    if verbose==10, return;end
  end

  tstart=tic();
  meshfile=gmsh.buildmesh3ds(geofile,N,'geodir',R.geodir,'meshdir',R.meshdir,'verbose',verbose);
  Th=siMesh(meshfile,'dim',3,'trans',Fmap);
  tcpu(1)=toc(tstart);
  if verbose>=1
    fprintf('*** Building/reading mesh file in %.3f(s)\n', tcpu(1))
    Th.info()
  end
  tstart=tic();
  Lop=Loperator(3,2,{1,[],[];[],1,[];[],[],1},[],c,a0);
  if isempty(u) % no exact solution
    RHSf=f;
  else
    cgradSu=@(x,y,z) sum(gradSu(x,y,z).*[c{1}(x,y,z);c{2}(x,y,z);c{3}(x,y,z)],1);
    if isnumeric(a0)
      RHSf=@(x,y,z) f(x,y,z)+cgradSu(x,y,z)+a0*u(x,y,z);
    else
      RHSf=@(x,y,z) f(x,y,z)+cgradSu(x,y,z)+a0(x,y,z).*u(x,y,z);
    end
  end
  pde=PDEelt(Lop,RHSf);
  bvp=BVP(Th,pde);
  tcpu(2)=toc(tstart);
  if verbose>=1
    fprintf('*** Setting surface BVP in %.3f(s)\n', tcpu(2))
  end
  info.u=u;
  info.f=RHSf;
  info.ndof=Th.nq;;
  info.tcpu=tcpu;
  info.N=N;
end